package main

import (
    "database/sql"
    "fmt"
    "log"
    "strconv"
    "net/http"

    "github.com/go-sql-driver/mysql"
    "github.com/gin-gonic/gin"
)

var db *sql.DB

func main() {
    // Conexión a la base de datos
    cfg := mysql.Config{
        User:   "root",
        Passwd: "marcelo",
        Net:    "tcp",
        Addr:   "127.0.0.1:3306",
        DBName: "tarea_1_sd",
    }
    // Manejo de errores para la conexión a nuestra base de datos
    var err error
    db, err = sql.Open("mysql", cfg.FormatDSN())
    if err != nil {
        log.Fatal(err)
    }

    pingErr := db.Ping()
    if pingErr != nil {
        log.Fatal(pingErr)
    }

    fmt.Println("Connected!")

    // Routing para los endpoints de nuestra API

    router := gin.Default()

    router.GET("/clientes/:id", getClientes)

    router.Run("127.0.0.1:5000")
}

// Estructuras de los objetos que vayamos a utilizar

type cliente struct {
    id_cliente int
    nombre string
    contrasena string
}

// Función para insertar en la BD un nuevo cliente

func addClient(cl cliente) (int64, error) {
    result, err := db.Exec("INSERT INTO cliente (nombre, contrasena) VALUES (?, ?)", cl.nombre, cl.contrasena)
    if err != nil {
        return 0, fmt.Errorf("addClient: %v", err)
    }
    id, err := result.LastInsertId()
    if err != nil {
        return 0, fmt.Errorf("addClient: %v", err)
    }
    return id, nil
}

// Función para la consulta a la BD acerca de un cliente con una id en específico

func clientByID(id int) (cliente, error) {
    // An album to hold data from the returned row.
    var clt cliente
    row := db.QueryRow("SELECT * FROM cliente WHERE id_cliente = ?", id)
    if err := row.Scan(&clt.id_cliente, &clt.nombre, &clt.contrasena); err != nil {
        if err == sql.ErrNoRows {
            return clt, fmt.Errorf("clientByID %d: no existe el cliente", id)
        }
        return clt, fmt.Errorf("clientByID %d: %v", id, err)
    }
    
    return clt, nil
}

// Endpoint para conseguir la respuesta de la base de datos acerca de un cliente en específico 

func getClientes(c *gin.Context) {
    id := c.Param("id")
    number,_ := strconv.Atoi(id)
    clt, err := clientByID(number)
    if err != nil {
        c.IndentedJSON(http.StatusNotFound, gin.H{"message": "cliente no encontrado"})
        return
    }

    // Serializar la información para que se pueda enviar como json
    data := map[string]interface{}{
		"id_cliente":    clt.id_cliente,
		"nombre":   clt.nombre,
		"contrasena": clt.contrasena,
	}
    c.JSON(http.StatusOK, data)
    
}